(ns leiningen.test.deps
  (:use [clojure.test]
        [leiningen.deps]
        [leiningen.test.helper :only [sample-project m2-dir native-project
                                      delete-file-recursively]])
  (:require [clojure.java.io :as io]
            [leiningen.core.main :as main]
            [leiningen.core.classpath :as classpath]
            [leiningen.core.eval :as eval]))

(deftest test-deps
  (let [sample-deps [["ring" "1.0.0"] ["rome" "0.9"] ["jdom" "1.0"]]]
    (doseq [[n v] sample-deps]
      (delete-file-recursively (m2-dir n v) :silently))
    (deps sample-project)
    (doseq [[n v] sample-deps]
      (is (.exists (m2-dir n v)) (str n " was not downloaded.")))))

(deftest test-snapshots-releases
  (let [pr (assoc sample-project :omit-default-repositories true
                  :repositories {"clojars" {:url "http://clojars.org/repo/"
                                            :snapshots false}})
        ps (assoc sample-project :omit-default-repositories true
                  :repositories {"clojars" {:url "http://clojars.org/repo/"
                                            :releases false}})
        slamhound ['slamhound "1.1.0-SNAPSHOT"]
        hooke ['robert/hooke "1.0.1"]
        deps (fn [project]
               (delete-file-recursively (apply m2-dir slamhound) :quiet)
               (delete-file-recursively (apply m2-dir hooke) :quiet)
               (leiningen.deps/deps project))]
    (deps (assoc pr :dependencies [hooke]))
    (is (.exists (m2-dir :robert/hooke "1.0.1")))
    (deps (assoc ps :dependencies [slamhound]))
    (is (.exists (m2-dir "slamhound" "1.1.0-SNAPSHOT")))
    (let [snaps-repo-rel-dep (assoc ps :dependencies [hooke])]
      (is (thrown? Exception (deps snaps-repo-rel-dep)))
      (is (not (.exists (m2-dir :robert/hooke "1.0.1")))))
    (let [rel-repo-snaps-dep (assoc pr :dependencies [slamhound])]
      (is (thrown? Exception (deps rel-repo-snaps-dep)))
      (is (not (.exists (m2-dir "slamhound" "1.1.0-SNAPSHOT"))))) ))

(def native-lib-files-map
  {:linux {:x86 #{"libjri.so" "libjinput-linux.so" "liblwjgl.so" "libopenal.so"
                  "librxtxSerial.so" "libjtokyocabinet.so"
                  "libjtokyocabinet.so.1" "libjtokyocabinet.so.1.1.0"
                  "libtokyocabinet.a" "libtokyocabinet.so"
                  "libtokyocabinet.so.9" "libtokyocabinet.so.9.10.0"
                  "libtokyocabinet.so.9.8.0"}
           :x86_64 #{"libjri.so" "libjinput-linux64.so" "liblwjgl64.so"
                     "libopenal64.so" "librxtxSerial.so" "libjtokyocabinet.so"
                     "libjtokyocabinet.so.1" "libjtokyocabinet.so.1.1.0"
                     "libtokyocabinet.a" "libtokyocabinet.so"
                     "libtokyocabinet.so.9" "libtokyocabinet.so.9.10.0"
                     "libtokyocabinet.so.9.8.0"}}
   :macosx {:x86 #{"libjri.jnilib" "libjinput-osx.jnilib" "liblwjgl.jnilib"
                   "openal.dylib" "librxtxSerial.jnilib"
                   "libjtokyocabinet.1.1.0.dylib" "libjtokyocabinet.1.dylib"
                   "libjtokyocabinet.dylib" "libjtokyocabinet.jnilib"
                   "libtokyocabinet.9.10.0.dylib"
                   "libtokyocabinet.9.8.0.dylib" "libtokyocabinet.9.dylib"
                   "libtokyocabinet.a" "libtokyocabinet.dylib"}
            :x86_64 #{"libjri.jnilib" "libjinput-osx.jnilib"
                      "liblwjgl.jnilib"
                      "openal.dylib" "librxtxSerial.jnilib"
                      "libjtokyocabinet.1.1.0.dylib"
                      "libjtokyocabinet.1.dylib"
                      "libjtokyocabinet.dylib" "libjtokyocabinet.jnilib"
                      "libtokyocabinet.9.10.0.dylib"
                      "libtokyocabinet.9.8.0.dylib" "libtokyocabinet.9.dylib"
                      "libtokyocabinet.a" "libtokyocabinet.dylib"}}
   :windows {:x86 #{"jri.dll" "rJava.dll" "jinput-dx8.dll" "jinput-raw.dll"
                    "lwjgl.dll" "OpenAL32.dll" "rxtxSerial.dll"}
             :x86_64 #{"jri.dll rJava.dll" "jinput-dx8_64.dll"
                       "jinput-raw_64.dll" "lwjgl64.dll" "OpenAL64.dll"
                       "rxtxSerial.dll"}}
   :solaris {:x86 #{"liblwjgl.so" "libopenal.so"}
             :x86_64 #{"liblwjgl64.so" "libopenal.so"}}})

(deftest ^:busted test-native-deps
  (delete-file-recursively (:native-path native-project) true)
  (deps native-project)
  (is (= (conj (get-in native-lib-files-map [(eval/get-os) (eval/get-arch)])
               ".gitkeep")
         (set (for [f (rest (file-seq (io/file (eval/native-arch-path
                                                native-project))))]
                (.getName f))))))
